import matplotlib.pyplot as plt
import numpy as np
import sys
import json
from argparse import ArgumentParser

def _read_file( filepath ):
    with open( filepath ) as fp:
        prj = fp.read()
        prjj = json.loads( prj )
    return prjj

def _get_channel_list( report_json ):
    all_keys = report_json["Channels"].keys()
    real_channels = set([key.split(":")[0] for key in all_keys]) 

    # Get the list of IP combo strings
    group_sub = list()
    for key in all_keys:
        if "Infected" in key: # just pick one that's guaranteed present. maybe pop would be better?
            group_sub.append( key.strip( "Infected:" ) )
    return real_channels, group_sub

def _get_values_for_key( key, group_sub ):
    ip_dict = {}
    for group in group_sub:
        ips = group.split(",")
        for ip_pair in ips:
            kvp = ip_pair.split( ":" )
            if kvp[0] not in ip_dict:
                ip_dict[kvp[0]] = set()
            ip_dict[kvp[0]].add( kvp[1] )

    values = ip_dict[key]
    print( values )
    return values

def _get_aggregated_values_for_channel( report_json, chan, key_to_agg_on, values, num_points ):
    prevs = {}
    for val in values:
        prevs[val] = None # np.zeros( num_points )
    for key in report_json["Channels"]:
        if key.startswith( chan + ":" ):
            for val in values: 
                compound = key_to_agg_on + ":" + val
                if compound in key:
                    if prevs[val] is None:
                        prevs[val] = np.zeros( num_points )
                    prevs[val] = np.add( prevs[val], report_json["Channels"][ key ][ "Data" ] )
    return prevs

def plot_channel_data_from_IP_key( filepath, channel, agg_key, norm=False ):
    # Read in the file
    prjj = _read_file( filepath )

    # Get the list of channels in the report
    real_channels, group_sub = _get_channel_list( prjj )
    channel = channel
    if channel not in real_channels:
        print( f"{channel} not found in {all_keys}." )
        sys.exit()
    print( real_channels ) # this could be made optional

    # Get the list of values for the aggregation key specified by the user.  
    values = _get_values_for_key( agg_key, group_sub )
    # need to generalize the line below
    num_points = len( prjj["Channels"][ "Infected:Age_Bin:Age_Bin_Property_From_0_To_20,QualityOfCare:High,QualityOfCare1:High,QualityOfCare2:High" ]["Data"] )

    pops = _get_aggregated_values_for_channel( prjj, "Statistical Population", agg_key, values, num_points )
    # Yes it's called prevs even though it can be anything
    prevs = _get_aggregated_values_for_channel( prjj, channel, agg_key, values, num_points )

    for val in values:
        if norm:
            prevs[val] = np.divide( prevs[val], pops[ val ] )
        plt.plot( prevs[val], label=val )
    plt.legend()
    plt.title( channel + "/" + agg_key )
    plt.show()


if __name__ == "__main__":
    parser = ArgumentParser(prog='plot_prop')
    parser.add_argument("-f", "--filepath", type=str, default=None)
    parser.add_argument("-k", "--agg_key", type=str, default=None)
    parser.add_argument("-c", "--channel", type=str, default="Infected")
    parser.add_argument("-n", "--norm", type=bool, default=False)
    args = parser.parse_args()

    if len(sys.argv) < 3:
        parser.print_help()
        sys.exit()

    plot_channel_data_from_IP_key( args.filepath.strip(), args.channel, args.agg_key, args.norm )


